@@ -118,13 +118,18 @@ void camv_draw_layer(camv_layer_t *ly, rnd_hid_gc_t gc, const rnd_hid_expose_ctx
 	}
 }
 
+rnd_cardinal_t camv_draw_inhibit = 0;
+
 void camv_expose_main(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd_xform_t *xform_caller)
 {
 	rnd_cardinal_t lid;
 	rnd_hid_gc_t gc;
 	int clearing = -1, direct = 1, need_flush = 0;
 	rnd_hid_t *save = rnd_render;
-	
+
+	if (camv_draw_inhibit)
+		return;
+
 	if ((rnd_render != NULL) && (!rnd_render->override_render))
 		rnd_render = hid;
 
